/*  __    __  __  __    __  ___
 * \  \  /  /    \  \  /  /  __/
 *  \  \/  /  /\  \  \/  /  /
 *   \____/__/  \__\____/__/
 *
 * Copyright 2014-2017 Vavr, http://vavr.io
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ch.usi.inf.sdm.sdm04.math;

import ch.usi.si.codelounge.jsicko.Contract;

import java.util.stream.DoubleStream;
import java.util.stream.IntStream;

@SuppressWarnings("unused") // methods used by jSicko
public interface IEEE754rUtilsContracts extends Contract {
    @Pure
    static boolean non_null_non_empty_arg(final double[] array) {
        return array != null && array.length > 0;
    }

    @Pure
    static boolean non_null_non_empty_arg_float(final float[] array) {
        return array != null && array.length > 0;
    }

    @Pure
    static boolean is_minimum_element(final double returns, final double[] array) {
        return DoubleStream.of(array).filter(value -> !Double.isNaN(value)).allMatch(e -> returns <= e);
    }

    @Pure
    static boolean is_maximum_element(final double returns, final double[] array) {
        return DoubleStream.of(array).filter(value -> !Double.isNaN(value)).allMatch(e -> returns >= e);
    }

    @Pure
    static boolean is_minimum_element_float(final float returns, final float[] array) {
        return is_minimum_element(returns, IntStream.range(0, array.length).mapToDouble(i -> array[i]).toArray());
    }

    @Pure
    static boolean is_maximum_element_float(final float returns, float[] array) {
        return is_maximum_element(returns, IntStream.range(0, array.length).mapToDouble(i -> array[i]).toArray());
    }

    @Pure
    static boolean is_minimum_element_2(final double returns, final double a, final double b) {
        return is_minimum_element(returns, new double[] { a, b });
    }

    @Pure
    static boolean is_minimum_element_3(final double returns, final double a, final double b, final double c) {
        return is_minimum_element(returns, new double[] { a, b, c });
    }

    @Pure
    static boolean is_minimum_element_2_float(final float returns, final float a, final float b) {
        return is_minimum_element(returns, new double[] { a, b });
    }

    @Pure
    static boolean is_minimum_element_3_float(final float returns, final float a, final float b, final float c) {
        return is_minimum_element(returns, new double[] { a, b, c });
    }

    @Pure
    static boolean is_maximum_element_2(final double returns, final double a, final double b) {
        return is_maximum_element(returns, new double[] { a, b });
    }

    @Pure
    static boolean is_maximum_element_3(final double returns, final double a, final double b, final double c) {
        return is_maximum_element(returns, new double[] { a, b, c });
    }

    @Pure
    static boolean is_maximum_element_2_float(final float returns, final float a, final float b) {
        return is_maximum_element(returns, new double[] { a, b });
    }

    @Pure
    static boolean is_maximum_element_3_float(final float returns, final float a, final float b, final float c) {
        return is_maximum_element(returns, new double[] { a, b, c });
    }
}
